import React, { Component } from 'react';
import { TouchableOpacity, Text, View } from 'react-native';
import PropTypes from 'prop-types';

import styleConstructor from './style';

class Day extends Component {
  static propTypes = {
    // TODO: disabled props should be removed
    state: PropTypes.oneOf(['disabled', 'today', '']),

    // Specify theme properties to override specific styles for calendar parts. Default = {}
    theme: PropTypes.object,
    marking: PropTypes.any,
    onPress: PropTypes.func,
    date: PropTypes.object,
    containerPriceStyle: PropTypes.any,
    textPriceStyle: PropTypes.any
  };

  constructor(props) {
    super(props);
    this.style = styleConstructor(props.theme);
    this.onDayPress = this.onDayPress.bind(this);
  }

  onDayPress() {
    this.props.onPress(this.props.date);
  }

  shouldComponentUpdate(nextProps) {
    const changed = ['state', 'children', 'marking', 'onPress'].reduce((prev, next) => {
      if (prev) {
        return prev;
      } else if (nextProps[next] !== this.props[next]) {
        return next;
      }
      return prev;
    }, false);
    if (changed === 'marking') {
      let markingChanged = false;
      if (this.props.marking && nextProps.marking) {
        markingChanged = !(
          this.props.marking.marked === nextProps.marking.marked &&
          this.props.marking.selected === nextProps.marking.selected &&
          this.props.marking.dotColor === nextProps.marking.dotColor &&
          this.props.marking.disabled === nextProps.marking.disabled
        );
      } else {
        markingChanged = true;
      }
      // console.log('marking changed', markingChanged);
      return markingChanged;
    } else {
      // console.log('changed', changed);
      return !!changed;
    }
  }

  render() {
    const containerStyle = [this.style.base];
    const textStyle = [this.style.text];
    const dotStyle = [this.style.dot];

    let marking = this.props.marking || {};
    if (marking && marking.constructor === Array && marking.length) {
      marking = {
        marking: true
      };
    }
    let dot;
    if (marking.marked) {
      dotStyle.push(this.style.visibleDot);
      if (marking.dotColor) {
        dotStyle.push({ backgroundColor: marking.dotColor });
      }
      dot = <View style={dotStyle} />;
    }

    if (marking.selected) {
      containerStyle.push(this.style.selected);
      dotStyle.push(this.style.selectedDot);
      textStyle.push(this.style.selectedText);
    } else if (typeof marking.disabled !== 'undefined' ? marking.disabled : this.props.state === 'disabled') {
      textStyle.push(this.style.disabledText);
    } else if (this.props.state === 'today') {
      textStyle.push(this.style.todayText);
    }
    return (
      <TouchableOpacity
        style={[{ alignItems: 'center' }, this.props.containerPriceStyle == null ? {} : this.props.containerPriceStyle]}
        onPress={this.onDayPress}
        disabled={typeof marking.disabled !== 'undefined' ? marking.disabled : this.props.state === 'disabled'}>
        <View style={containerStyle}>
          <Text style={textStyle}>{String(this.props.children)}</Text>
          {dot}
        </View>
        <View>
          <Text style={this.props.textPriceStyle == null ? {} : this.props.textPriceStyle}>{this.props.price.toString().slice(0, -3)}</Text>
        </View>
      </TouchableOpacity>
    );
  }
}

export default Day;
